@@ -17,10 +17,12 @@
 #include "smbios.h"
 #include "klog.h"
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- * bios info
+/**
+ * @defgroup BIOSint BIOS information
+ * @ingroup  libhdINFOint
+ * @brief BIOS information scan
  *
- * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ * @{
  */
 
 #if defined(__i386__) || defined (__x86_64__) || defined (__ia64__)
@@ -66,7 +68,13 @@ static struct {
   {   0,   0, 1600, 1200, "Dell Computer Corporation", "Inspiron 8200", NULL },
   {   0,   0, 1600, 1200, "Dell Computer Corporation", "Latitude C840", NULL },
   { 190, 110, 1024,  600, "FUJITSU SIEMENS", "LIFEBOOK P1510", NULL },
-  { 285, 215, 1024,  768, "FUJITSU SIEMENS", "LIFEBOOK S7020", NULL }
+  { 285, 215, 1024,  768, "FUJITSU SIEMENS", "LIFEBOOK S7020", NULL },
+  { 305, 230, 1400, 1050, "Samsung Electronics", "SX20S", "Revision PR" },
+  { 290, 200, 1280,  800, "TOSHIBA", "Satellite M30X", "PSA72E-00J020GR" },
+  { 305, 230, 1400, 1050, "ASUSTeK Computer Inc.", "M6V", "1.0" },
+  { 246, 185, 1024,  768, "FUJITSU SIEMENS", "LifeBook T Series", NULL },
+  { 304, 190, 1400, 1050, "Hewlett-Packard", "HP Compaq nc6000 (PN887PA#ABG)", "F.0F" },
+  { 332, 208, 1280,  800, "MICRO-STAR INT'L CO.,LTD.", "MS-1011", "0101" },
 };
 
 #define BIOS_TEST
@@ -612,24 +620,37 @@ void smbios_get_info(hd_data_t *hd_data, memory_range_t *mem, bios_info_t *bt)
   memory_range_t memory;
   hd_smbios_t *sm;
 
-  if(!mem->data || mem->size < 0x100) return;
+  if(!mem->data || mem->size < 0x10) return;
 
-  for(u = ok = 0; u <= mem->size - 0x100; u += 0x10) {
+  for(u = ok = 0; u <= mem->size - 0x10; u += 0x10) {
     if(*(unsigned *) (mem->data + u) == 0x5f4d535f) {	/* "_SM_" */
       hlen = mem->data[u + 5];
+      if(hlen < 0x1e || u + hlen > mem->size) continue;
       addr = *(unsigned *) (mem->data + u + 0x18);
       len = *(unsigned short *) (mem->data + u + 0x16);
       structs = *(unsigned short *) (mem->data + u + 0x1c);
-      if(hlen < 0x1e) continue;
       ok = crc(mem->data + u, hlen) == 0 && len;
-      if(ok) break;
+      if(ok) {
+        bt->smbios_ver = (mem->data[u + 6] << 8) + mem->data[u + 7];
+        break;
+      }
+    }
+    /* Also look for legacy DMI entry point */
+    if(memcmp(mem->data + u, "_DMI_", 5) == 0) {
+      hlen = 0x0f;
+      addr = *(unsigned *) (mem->data + u + 0x08);
+      len = *(unsigned short *) (mem->data + u + 0x06);
+      structs = *(unsigned short *) (mem->data + u + 0x0c);
+      ok = crc(mem->data + u, hlen) == 0 && len;
+      if(ok) {
+        bt->smbios_ver = ((mem->data[u + 0x0e] & 0xf0) << 4) + (mem->data[u + 0x0e] & 0x0f);
+        break;
+      }
     }
   }
 
   if(!ok) return;
 
-  bt->smbios_ver = (mem->data[u + 6] << 8) + mem->data[u + 7];
-
   hd_data->smbios = smbios_free(hd_data->smbios);
 
   memory.start = mem->start + u;
@@ -791,7 +812,7 @@ void add_panel_info(hd_data_t *hd_data, bios_info_t *bt)
   hd_smbios_t *sm;
   unsigned u;
 
-  if(bt->lcd.width || !hd_data->smbios) return;
+  if(!hd_data->smbios) return;
 
   vendor = name = version = NULL;
   width = height = 0;
@@ -1035,11 +1056,9 @@ int get_bios32_info(hd_data_t *hd_data, memory_range_t *mem, bios32_info_t *bios
   return ok;
 }
 
-/*
- *
+/**
  * db format (32 bits):
  * leaf: 1, last: 1, key: 6, range: 4, ofs: 20
- *
  */
 int tp_lookup(char *key_str, unsigned *width, unsigned *height, unsigned *xsize, unsigned *ysize)
 {
@@ -1088,3 +1107,5 @@ int tp_lookup(char *key_str, unsigned *width, unsigned *height, unsigned *xsize,
 
 #endif /* defined(__i386__) || defined (__x86_64__) */
 
+/** @} */
+
