@@ -11,6 +11,14 @@
 #include "isdn.h"
 #include "hddb_int.h"
 
+/**
+ * @defgroup HDDBint Hardware DB (HDDB)
+ * @ingroup libhdInternals
+ * @brief Hardware DB functions
+ *
+ * @{
+ */
+
 extern hddb2_data_t hddb_internal;
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -34,10 +42,13 @@ typedef struct line_s {
 
 typedef struct {
   int len;
-  unsigned val[32];	/* arbitrary (approx. max. number of modules/xf86 config lines) */
+  unsigned val[32];	/**< arbitrary (approx. max. number of modules/xf86 config lines) */
 } tmp_entry_t;
 
-/* except for driver, all strings are static and _must not_ be freed */
+/**
+ * Hardware DB search struct.
+ * @note except for driver, all strings are static and _must not_ be freed
+ */
 typedef struct {
   hddb_entry_mask_t key;
   hddb_entry_mask_t value;
@@ -58,7 +69,6 @@ typedef struct {
   unsigned hwclass;
 } hddb_search_t;
 
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 static void hddb_init_pci(hd_data_t *hd_data);
 static char *get_mi_field(char *str, char *tag, int field_len, unsigned *value, unsigned *has_value);
@@ -163,7 +173,7 @@ modinfo_t *parse_modinfo(str_list_t *file)
   unsigned u;
   char alias[256], module[256];
 
-  /* lenght + 1! */
+  /* length + 1! */
   for(len = 1, sl = file; sl; sl = sl->next) len++;
 
   modinfo = new_mem(len * sizeof *modinfo);
@@ -171,9 +181,15 @@ modinfo_t *parse_modinfo(str_list_t *file)
   for(m = modinfo, sl = file; sl; sl = sl->next) {
     if(sscanf(sl->str, "alias %255s %255s", alias, module) != 2) continue;
 
+    m->module = new_str(module);
+    m->alias = new_str(alias);
+    m->type = mi_other;
+
     if(!strncmp(alias, "pci:", sizeof "pci:" - 1)) {
       s = alias + sizeof "pci:" - 1;
 
+      m->type = mi_pci;
+
       if(!(s = get_mi_field(s, "v", 8, &m->pci.vendor, &u))) continue;
       m->pci.has.vendor = u;
 
@@ -194,34 +210,9 @@ modinfo_t *parse_modinfo(str_list_t *file)
 
       if(!(s = get_mi_field(s, "i", 2, &m->pci.prog_if, &u))) continue;
       m->pci.has.prog_if = u;
-
-      m->module = new_str(module);
-      m->alias = new_str(alias);
-      m++->type = mi_pci;
-
-      continue;
-    }
-
-    if(!strncmp(alias, "usb:", sizeof "usb:" - 1)) {
-      s = alias + sizeof "usb:" - 1;
-
-      m->module = new_str(module);
-      m->alias = new_str(alias);
-      m++->type = mi_usb;
-
-      continue;
-    }
-
-    if(!strncmp(alias, "pcmcia:", sizeof "pcmcia:" - 1)) {
-      s = alias + sizeof "pcmcia:" - 1;
-
-      m->module = new_str(module);
-      m->alias = new_str(alias);
-      m++->type = mi_pcmcia;
-
-      continue;
     }
 
+    m++;
   }
 
   /* note: list stops at first entry with m->type == mi_none */
@@ -231,8 +222,8 @@ modinfo_t *parse_modinfo(str_list_t *file)
   for(m = modinfo; m->type; m++) {
     switch(m->type) {
       case mi_pci:
-        fprintf(stderr, "%s:%d, %s\n  v 0x%x:%u, d 0x%x:%u, sv 0x%x:%u, sd 0x%x:%u, bc 0x%x:%u, sc 0x%x:%u, i 0x%x:%u\n",
-          m->module, m->type, m->alias,
+        fprintf(stderr, "%s: %s\n  v 0x%x:%u, d 0x%x:%u, sv 0x%x:%u, sd 0x%x:%u, bc 0x%x:%u, sc 0x%x:%u, i 0x%x:%u\n",
+          m->module, m->alias,
           m->pci.vendor, m->pci.has.vendor,
           m->pci.device, m->pci.has.device,
           m->pci.sub_vendor, m->pci.has.sub_vendor,
@@ -243,10 +234,9 @@ modinfo_t *parse_modinfo(str_list_t *file)
         );
         break;
 
-      case mi_usb:
-      case mi_pcmcia:
-        fprintf(stderr, "%s:%d, %s\n",
-          m->module, m->type, m->alias
+      case mi_other:
+        fprintf(stderr, "%s: %s\n",
+          m->module, m->alias
         );
         break;
 
@@ -260,7 +250,9 @@ modinfo_t *parse_modinfo(str_list_t *file)
 }
 
 
-/* return prio, 0: no match */
+/**
+ *  return prio, 0: no match 
+ */
 int match_modinfo(modinfo_t *db, modinfo_t *match)
 {
   int prio = 0;
@@ -335,8 +327,7 @@ int match_modinfo(modinfo_t *db, modinfo_t *match)
       }
       break;
 
-    case mi_usb:
-    case mi_pcmcia:
+    case mi_other:
       if(match->alias && db->alias) {
         if(!fnmatch(db->alias, match->alias, 0)) {
           s = strchr(db->alias, '*');
@@ -357,7 +348,7 @@ driver_info_t *hd_modinfo_db(hd_data_t *hd_data, modinfo_t *modinfo_db, hd_t *hd
 {
   driver_info_t **di = NULL, *di2;
   pci_t *pci;
-  char *mod_list[16 /* arbitrary */];
+  char *mod_list[16 /* arbitrary, > 0 */];
   int mod_prio[sizeof mod_list / sizeof *mod_list];
   int i, prio, mod_list_len;
   modinfo_t match = { };
@@ -370,18 +361,8 @@ driver_info_t *hd_modinfo_db(hd_data_t *hd_data, modinfo_t *modinfo_db, hd_t *hd
   ) {
     match.type = mi_pci;
   }
-  else if(
-    ID_TAG(hd->vendor.id) == TAG_USB ||
-    ID_TAG(hd->device.id) == TAG_USB
-  ) {
-    match.type = mi_usb;
-  }
-  else if(
-    ID_TAG(hd->vendor.id) == TAG_PCMCIA ||
-    ID_TAG(hd->device.id) == TAG_PCMCIA ||
-    (hd->modalias && !strncmp(hd->modalias, "pcmcia:", sizeof "pcmcia:" - 1))
-  ) {
-    match.type = mi_pcmcia;
+  else {
+    match.type = mi_other;
   }
 
   match.alias = hd->modalias;
@@ -435,7 +416,7 @@ driver_info_t *hd_modinfo_db(hd_data_t *hd_data, modinfo_t *modinfo_db, hd_t *hd
           di2->any.type == di_module &&
           di2->any.hddb0 &&
           di2->any.hddb0->str &&
-          !strcmp(di2->any.hddb0->str, modinfo_db->module)
+          !hd_mod_cmp(di2->any.hddb0->str, modinfo_db->module)
         ) break;
       }
 
@@ -457,6 +438,11 @@ driver_info_t *hd_modinfo_db(hd_data_t *hd_data, modinfo_t *modinfo_db, hd_t *hd
     }
   }
 
+  if(!mod_list_len && hd->modalias && !strchr(hd->modalias, ':')) {
+    mod_prio[mod_list_len] = 0;
+    mod_list[mod_list_len++] = hd->modalias;
+  }
+
   for(prio = 256; prio >= 0; prio--) {
     for(i = 0; i < mod_list_len; i++) {
       if(mod_prio[i] == prio) {
@@ -1743,7 +1729,7 @@ void test_db(hd_data_t *hd_data)
 #endif
 
 
-str_list_t *get_hddb_packages(hd_data_t *hd_data)
+str_list_t *hddb_get_packages(hd_data_t *hd_data)
 {
   return NULL;
 }
@@ -2411,6 +2397,17 @@ driver_info_t *monitor_driver(hd_data_t *hd_data, hd_t *hd)
     ddi->max_vsync = mi->max_vsync;
     ddi->min_hsync = mi->min_hsync;
     ddi->max_hsync = mi->max_hsync;
+    ddi->bandwidth = mi->clock / 1000;
+    ddi->hdisp     = mi->hdisp; 
+    ddi->hsyncstart= mi->hsyncstart;
+    ddi->hsyncend  = mi->hsyncend;
+    ddi->htotal    = mi->htotal;
+    ddi->hflag     = mi->hflag;
+    ddi->vdisp     = mi->vdisp;
+    ddi->vsyncstart= mi->vsyncstart;
+    ddi->vsyncend  = mi->vsyncend;
+    ddi->vtotal    = mi->vtotal;
+    ddi->vflag     = mi->vflag;
 
     for(res = hd->res; res; res = res->next) {
       if(res->any.type == res_monitor) {
@@ -3038,3 +3035,5 @@ char *hid_tag_name2(int tag)
   return (unsigned) tag < sizeof hid_tag_names2 / sizeof *hid_tag_names2 ? hid_tag_names2[tag] : "";
 }
 
+/** @} */
+
