@@ -11,11 +11,15 @@
 #include "smbios.h"
 
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+/**
+ * @defgroup HDPRINTint Hardware scan console output
+ * @ingroup libhdInternals
+ * @brief Hardware scan console output functions
+ *
  * This module provides a function that prints a hardware entry. 
  * This is useful for debugging or to provide the user with some fancy info.
  *
- * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ * @{
  */
 
 #ifndef LIBHD_TINY
@@ -467,6 +471,12 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
     s = free_mem(s);
   }
 
+  if(h->driver_modules) {
+    s = hd_join("\", \"", h->driver_modules);
+    dump_line("Driver Modules: \"%s\"\n", s);
+    s = free_mem(s);
+  }
+
   if(h->broken) {
     dump_line_str("Warning: might be broken\n");
   }
@@ -525,8 +535,9 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
 
   if(
     h->is.zip ||
-    h->is.cdr || h->is.cdrw || h->is.dvd ||
-    h->is.dvdr || h->is.dvdram || h->is.pppoe || h->is.wlan
+    h->is.cdr || h->is.cdrw || h->is.dvd || h->is.dvdr || h->is.dvdrw ||
+    h->is.dvdpr || h->is.dvdprw || h->is.dvdprdl || h->is.dvdram ||
+    h->is.pppoe || h->is.wlan || h->is.hotpluggable
   ) {
     dump_line_str("Features:");
     i = 0;
@@ -535,9 +546,14 @@ void dump_normal(hd_data_t *hd_data, hd_t *h, FILE *f)
     if(h->is.cdrw) dump_line0("%s CD-RW", i++ ? "," : "");
     if(h->is.dvd) dump_line0("%s DVD", i++ ? "," : "");
     if(h->is.dvdr) dump_line0("%s DVD-R", i++ ? "," : "");
+    if(h->is.dvdrw) dump_line0("%s DVD-RW", i++ ? "," : "");
+    if(h->is.dvdpr) dump_line0("%s DVD+R", i++ ? "," : "");
+    if(h->is.dvdprw) dump_line0("%s DVD+RW", i++ ? "," : "");
+    if(h->is.dvdprdl) dump_line0("%s DVD+DL", i++ ? "," : "");
     if(h->is.dvdram) dump_line0("%s DVDRAM", i++ ? "," : "");
     if(h->is.pppoe) dump_line0("%s PPPOE", i++ ? "," : "");
     if(h->is.wlan) dump_line0("%s WLAN", i++ ? "," : "");
+    if(h->is.hotpluggable) dump_line0("%s Hotpluggable", i++ ? "," : "");
     dump_line0("\n");
   }
 
@@ -1297,3 +1313,5 @@ void hd_dump_entry(hd_data_t *hd_data, hd_t *h, FILE *f) { }
 
 #endif	/* ifndef LIBHD_TINY */
 
+/** @} */
+
