@@ -10,10 +10,12 @@
 #include "hd_int.h"
 #include "int.h"
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
- * internal things
+/**
+ * @defgroup LIBHDint Internal utilities
+ * @ingroup libhdInternals
+ * @brief Internal utility functions
  *
- * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ * @{
  */
 
 static void int_hotplug(hd_data_t *hd_data);
@@ -42,6 +44,7 @@ static void int_system(hd_data_t *hd_data);
 static void int_legacy_geo(hd_data_t *hd_data);
 #endif
 static void int_find_parent(hd_data_t *hd_data);
+static void int_add_driver_modules(hd_data_t *hd_data);
 
 
 void hd_scan_int(hd_data_t *hd_data)
@@ -84,6 +87,9 @@ void hd_scan_int(hd_data_t *hd_data)
   }
   hd_data->flags.keep_kmods = 0;
 
+  PROGRESS(7, 1, "modules");
+  int_add_driver_modules(hd_data);
+
   PROGRESS(8, 0, "usbscsi");
   int_fix_usb_scsi(hd_data);
 
@@ -120,11 +126,15 @@ void hd_scan_int(hd_data_t *hd_data)
 void int_hotplug(hd_data_t *hd_data)
 {
   hd_t *hd;
+  hal_prop_t *prop;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
     if(hd->bus.id == bus_usb || hd->usb_guid) {
       hd->hotplug = hp_usb;
     }
+    if((prop = hal_get_bool(hd->hal_prop, "storage.hotpluggable")) && prop->val.b) {
+      hd->is.hotpluggable = 1;
+    }
   }
 }
 
@@ -134,16 +144,41 @@ void int_hotplug(hd_data_t *hd_data)
 void int_cdrom(hd_data_t *hd_data)
 {
   hd_t *hd;
+  hal_prop_t *prop;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
     if(
       hd->base_class.id == bc_storage_device &&
-      hd->sub_class.id == sc_sdev_cdrom &&
-      !hd->prog_if.id
+      hd->sub_class.id == sc_sdev_cdrom
     ) {
-      if(hd->device.name && strstr(hd->device.name, "DVD")) {
-        hd->prog_if.id = 3;
+      if(!hd->prog_if.id && hd->device.name && strstr(hd->device.name, "DVD")) hd->is.dvd = 1;
+
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.cdr")) && prop->val.b) {
+        hd->is.cdr = 1;
       }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.cdrw")) && prop->val.b) {
+        hd->is.cdrw = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdr")) && prop->val.b) {
+        hd->is.dvdr = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdrw")) && prop->val.b) {
+        hd->is.dvdrw = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdram")) && prop->val.b) {
+        hd->is.dvdram = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdplusr")) && prop->val.b) {
+        hd->is.dvdpr = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdplusrw")) && prop->val.b) {
+        hd->is.dvdprw = 1;
+      }
+      if((prop = hal_get_bool(hd->hal_prop, "storage.cdrom.dvdplusrdl")) && prop->val.b) {
+        hd->is.dvdprdl = 1;
+      }
+
+      if(hd->is.dvd) hd->prog_if.id = 3;
     }
   }
 }
@@ -738,6 +773,7 @@ void int_wlan(hd_data_t *hd_data)
   str_list_t *sl;
   unsigned u, found;
   static char *wlan_mods[] = {
+    "acx",
     "acx_pci",
     "airo",
     "airo_cs",
@@ -820,7 +856,6 @@ void int_udev(hd_data_t *hd_data)
 {
   hd_udevinfo_t *ui;
   hd_t *hd;
-  char *s = NULL;
   str_list_t *sl;
 
   if(!hd_data->udevinfo) read_udevinfo(hd_data);
@@ -828,33 +863,52 @@ void int_udev(hd_data_t *hd_data)
   if(!hd_data->udevinfo) return;
 
   for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(!hd->unix_dev_names && hd->unix_dev_name) {
+      add_str_list(&hd->unix_dev_names, hd->unix_dev_name);
+    }
+
     if(!hd->sysfs_id) continue;
 
     for(ui = hd_data->udevinfo; ui; ui = ui->next) {
       if(ui->name && !strcmp(ui->sysfs, hd->sysfs_id)) {
-        hd->unix_dev_names = free_str_list(hd->unix_dev_names);
-        hd->unix_dev_name = free_mem(hd->unix_dev_name);
-        str_printf(&s, 0, "/dev/%s", ui->name);
-        add_str_list(&hd->unix_dev_names, s);
+        if(!search_str_list(hd->unix_dev_names, ui->name)) {
+          add_str_list(&hd->unix_dev_names, ui->name);
+        }
         for(sl = ui->links; sl; sl = sl->next) {
-          str_printf(&s, 0, "/dev/%s", sl->str);
-          add_str_list(&hd->unix_dev_names, s);
-        }
-        s = free_mem(s);
-
-        sl = hd->unix_dev_names;
-
-        if(hd_data->flags.udev) {
-          /* use first link as canonical device name */
-          if(ui->links) sl = sl->next;
+          if(!search_str_list(hd->unix_dev_names, sl->str)) {
+            add_str_list(&hd->unix_dev_names, sl->str);
+          }
         }
 
-        hd->unix_dev_name = new_str(sl->str);
+        if(!hd->unix_dev_name) {
+          sl = hd->unix_dev_names;
+
+          if(hd_data->flags.udev) {
+            /* use first link as canonical device name */
+            if(ui->links) sl = sl->next;
+          }
+
+          hd->unix_dev_name = new_str(sl->str);
+        }
 
         break;
       }
     }
   }
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    if(!hd->unix_dev_names) continue;
+
+    for(ui = hd_data->udevinfo; ui; ui = ui->next) {
+      if(search_str_list(hd->unix_dev_names, ui->name)) {
+        for(sl = ui->links; sl; sl = sl->next) {
+          if(!search_str_list(hd->unix_dev_names, sl->str)) {
+            add_str_list(&hd->unix_dev_names, sl->str);
+          }
+        }
+      }
+    }
+  }
 }
 
 
@@ -1106,3 +1160,27 @@ void int_find_parent(hd_data_t *hd_data)
 }
 
 
+void int_add_driver_modules(hd_data_t *hd_data)
+{
+  hd_t *hd;
+  hd_sysfsdrv_t *sf;
+  str_list_t *sl;
+
+  hd_sysfs_driver_list(hd_data);
+
+  for(hd = hd_data->hd; hd; hd = hd->next) {
+    hd->driver_modules = free_str_list(hd->driver_modules);
+
+    for(sl = hd->drivers; sl; sl = sl->next) {
+      for(sf = hd_data->sysfsdrv; sf; sf = sf->next) {
+        if(sf->module && !strcmp(sf->driver, sl->str)) {
+          add_str_list(&hd->driver_modules, sf->module);
+        }
+      }
+    }
+  }
+}
+
+
+/** @} */
+
